@@ -1,8 +1,8 @@
 use pyo3::intern;
 use pyo3::prelude::*;
-use pyo3::sync::GILOnceCell;
+use pyo3::sync::PyOnceLock;
 
-static MISSING_SENTINEL_OBJECT: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+static MISSING_SENTINEL_OBJECT: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
 
 pub fn get_missing_sentinel_object(py: Python<'_>) -> &Bound<'_, PyAny> {
     MISSING_SENTINEL_OBJECT
