@@ -224,6 +224,7 @@ pub struct InternalValidator {
     extra_behavior: Option<ExtraBehavior>,
     from_attributes: Option<bool>,
     context: Option<Py<PyAny>>,
+    field_name: Option<Py<PyString>>,
     self_instance: Option<Py<PyAny>>,
     recursion_guard: RecursionState,
     pub(crate) exactness: Option<Exactness>,
@@ -257,6 +258,7 @@ impl InternalValidator {
             extra_behavior: extra.extra_behavior,
             from_attributes: extra.from_attributes,
             context: extra.context.map(|d| d.clone().unbind()),
+            field_name: extra.field_name.as_ref().map(|d| d.clone().unbind()),
             self_instance: extra.self_instance.map(|d| d.clone().unbind()),
             recursion_guard: state.recursion_guard.clone(),
             exactness: state.exactness,
@@ -321,7 +323,7 @@ impl InternalValidator {
             strict: self.strict,
             extra_behavior: self.extra_behavior,
             from_attributes: self.from_attributes,
-            field_name: None,
+            field_name: self.field_name.as_ref().map(|d| d.bind(py).clone()),
             context: self.context.as_ref().map(|data| data.bind(py)),
             self_instance: self.self_instance.as_ref().map(|data| data.bind(py)),
             cache_str: self.cache_str,
