@@ -1,4 +1,5 @@
-use pyo3::exceptions::PyNotImplementedError;
+use std::sync::Arc;
+
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyType};
@@ -12,7 +13,7 @@ use super::{BuildValidator, CombinedValidator, DefinitionsBuilder, ValidationSta
 
 #[derive(Debug, Clone)]
 pub struct IsInstanceValidator {
-    class: PyObject,
+    class: Py<PyAny>,
     class_repr: String,
     name: String,
 }
@@ -23,8 +24,8 @@ impl BuildValidator for IsInstanceValidator {
     fn build(
         schema: &Bound<'_, PyDict>,
         _config: Option<&Bound<'_, PyDict>>,
-        _definitions: &mut DefinitionsBuilder<CombinedValidator>,
-    ) -> PyResult<CombinedValidator> {
+        _definitions: &mut DefinitionsBuilder<Arc<CombinedValidator>>,
+    ) -> PyResult<Arc<CombinedValidator>> {
         let py = schema.py();
         let cls_key = intern!(py, "cls");
         let class = schema.get_as_req(cls_key)?;
@@ -37,11 +38,11 @@ impl BuildValidator for IsInstanceValidator {
 
         let class_repr = class_repr(schema, &class)?;
         let name = format!("{}[{class_repr}]", Self::EXPECTED_TYPE);
-        Ok(Self {
+        Ok(CombinedValidator::IsInstance(Self {
             class: class.into(),
             class_repr,
             name,
-        }
+        })
         .into())
     }
 }
@@ -54,12 +55,16 @@ impl Validator for IsInstanceValidator {
         py: Python<'py>,
         input: &(impl Input<'py> + ?Sized),
         _state: &mut ValidationState<'_, 'py>,
-    ) -> ValResult<PyObject> {
+    ) -> ValResult<Py<PyAny>> {
         let Some(obj) = input.as_python() else {
-            return Err(ValError::InternalErr(PyNotImplementedError::new_err(
-                "Cannot check isinstance when validating from json, \
-                            use a JsonOrPython validator instead.",
-            )));
+            let method_name = "isinstance".to_string();
+            return Err(ValError::new(
+                ErrorType::NeedsPythonObject {
+                    context: None,
+                    method_name,
+                },
+                input,
+            ));
         };
         match obj.is_instance(self.class.bind(py))? {
             true => Ok(obj.clone().unbind()),
@@ -81,7 +86,7 @@ impl Validator for IsInstanceValidator {
 pub fn class_repr(schema: &Bound<'_, PyDict>, class: &Bound<'_, PyAny>) -> PyResult<String> {
     match schema.get_as(intern!(schema.py(), "cls_repr"))? {
         Some(s) => Ok(s),
-        None => match class.extract::<&PyType>() {
+        None => match class.downcast::<PyType>() {
             Ok(t) => Ok(t.qualname()?.to_string()),
             Err(_) => Ok(class.repr()?.extract()?),
         },
