@@ -1,5 +1,3 @@
-from typing import List
-
 import pytest
 
 from pydantic_core import PydanticUndefined, SchemaValidator, ValidationError, core_schema
@@ -8,7 +6,7 @@ from pydantic_core import PydanticUndefined, SchemaValidator, ValidationError, c
 def test_model_root():
     class RootModel:
         __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
-        root: List[int]
+        root: list[int]
 
     v = SchemaValidator(
         core_schema.model_schema(RootModel, core_schema.list_schema(core_schema.int_schema()), root_model=True)
@@ -36,7 +34,7 @@ def test_model_root():
 def test_revalidate():
     class RootModel:
         __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
-        root: List[int]
+        root: list[int]
 
     v = SchemaValidator(
         core_schema.model_schema(
@@ -139,20 +137,22 @@ def test_field_function():
 
     v = SchemaValidator(
         core_schema.model_schema(
-            RootModel, core_schema.field_after_validator_function(f, 'root', core_schema.str_schema()), root_model=True
+            RootModel,
+            core_schema.with_info_after_validator_function(f, core_schema.str_schema()),
+            root_model=True,
         )
     )
     m = v.validate_python('foobar', context='call 1')
     assert isinstance(m, RootModel)
     assert m.root == 'foobar validated'
-    assert call_infos == ["FieldValidationInfo(config=None, context='call 1', field_name='root')"]
+    assert call_infos == ["ValidationInfo(config=None, context='call 1', data=None, field_name='root')"]
 
     m2 = v.validate_assignment(m, 'root', 'baz', context='assignment call')
     assert m2 is m
     assert m.root == 'baz validated'
     assert call_infos == [
-        "FieldValidationInfo(config=None, context='call 1', field_name='root')",
-        "FieldValidationInfo(config=None, context='assignment call', field_name='root')",
+        "ValidationInfo(config=None, context='call 1', data=None, field_name='root')",
+        "ValidationInfo(config=None, context='assignment call', data=None, field_name='root')",
     ]
 
 
